@@ -511,7 +511,7 @@ def previous_peak(waves, chan_path, unit, n_chans = 20):
     max_chan_path = list(Path(chan_path/'routinesMemory').glob(f'dsm_{unit}_peakchan*'))[0]
     max_chan = int(np.load(max_chan_path))
    # waves = waves.T
-    if max_chan < n_chans - 1:
+    if max_chan <= n_chans - 1:
         bounds = (0, max_chan+n_chans +1)
     elif max_chan > 384 - n_chans -1:
         bounds = (max_chan-n_chans, 384)
@@ -544,20 +544,7 @@ def previous_peak(waves, chan_path, unit, n_chans = 20):
     # find the max amd argmax of pbp
     argmax_pbp = np.argmax(pbp)
     max_pbp = pbp[argmax_pbp]
-    # get the values before the peak
-    before_max = pbp[:argmax_pbp]
-    # get the values after
-    after_max = pbp[argmax_pbp+1:]
-    # find the crossing points with the fn below
-
-    if len(before_max) > 0 and len(after_max) > 0:
-        before_half_chan = np.where(before_max < max_pbp * 0.5)[0][-1]
-        after_half_chan = np.where(after_max < max_pbp * 0.5)[0][0]
-#        print(before_half_chan, after_half_chan, argmax_pbp)
-        spread = argmax_pbp - before_half_chan + after_half_chan - 1
-    else:
-        spread = 0
-    return pbp, spread, max_pbp
+    return pbp, max_pbp
 
 def consecutive_peaks_amp(mean_waves: np.array) -> tuple:
 
@@ -654,7 +641,6 @@ def chan_spread(all_wav, chan_path, unit, n_chans = 20, chan_spread_dist = 25.6)
         bounds = (max_chan-n_chans, 384)
     else:
         bounds = (max_chan-n_chans, max_chan+n_chans+1)
-    print(max_chan)
     bound_dist = dists[bounds[0]:bounds[1]+1]
     bound_p2p = p2p[bounds[0]:bounds[1]+1]
     bound_dist_p2p = dist_p2p[bounds[0]:bounds[1]+1]
@@ -679,7 +665,6 @@ def wvf_shape(wave):
     peak_trough, most_neg, start_thres, end_thres, peak_order = (False,) * 5
 
     # first get the length of the waveform so we can determine length of the end
-
     if wave.shape[0] == 82:
         end_section = 10
     elif wave.shape[0] == 8200:
@@ -898,14 +883,16 @@ def waveform_features(all_waves, dpath,  peak_chan, unit):
     best_wave = all_waves[peak_chan].reshape(1,-1)
     best_wave -= np.mean(best_wave[:20])
     best_wave = interp_wave(best_wave).reshape(-1)
+
+    if not wvf_shape(best_wave):
+        return list(np.zeros(17))
+    # get positive peak
+
     peak_t, peak_v = detect_peaks(best_wave)
     neg_id = np.argmin(peak_v)
     neg_v = peak_v[neg_id]
     neg_t = peak_t[neg_id]
 
-    if not wvf_shape(best_wave):
-        return np.zeros(17)
-    # get positive peak
     if neg_id + 1 <= peak_t.shape:
         pos_v = peak_v[neg_id + 1]
         pos_t = peak_t[neg_id + 1]
@@ -940,7 +927,7 @@ def waveform_features(all_waves, dpath,  peak_chan, unit):
     _,_,_,_,_, chans = chan_spread(all_waves, dpath, unit)
 
     #backprop spread
-    _, bp_spread, backp_max =  previous_peak(all_waves, dpath, unit)
+    _, backp_max =  previous_peak(all_waves, dpath, unit)
 
     ret_arr = [unit, neg_v, neg_t, pos_v, pos_t,pos_10_90_t,
         neg_10_90_t, pos50, neg50, onset_t, onset_amp, wvfd, ptr, coeff1[0], coeff2[0], chans, backp_max]
@@ -1002,7 +989,7 @@ def chan_spread_bp_plot(dp, unit, n_chans=20):
 
         if n_chans %2 !=0: n_chans +=1
         all_waves_unit_x = np.load(curr_fil)
-        backp, bp_spread, true_bp =  previous_peak(all_waves_unit_x.T, dp, unit, n_chans)
+        backp, true_bp =  previous_peak(all_waves_unit_x.T, dp, unit, n_chans)
         csp_x = chan_spread(all_waves_unit_x.T,dp, unit, n_chans)
         peak_chan = csp_x[0]
         print(peak_chan)
@@ -1067,13 +1054,17 @@ def temp_feat(dp, units, use_or_operator = True, use_consecutive = False):
 
     # units can be either a single integer or a list or np array of units
 
-    if isinstance(units, int):
+    if isinstance(units, (int, np.int16, np.int32, np.int64)):
         units = [units]
 
     all_ft_list = []
     for unit in units:
+
         unit_spikes = trn_filtered(dp, unit, use_or_operator = use_or_operator, use_consecutive = use_consecutive)
-        all_ft_list.append(temporal_features(dp,unit_spikes, unit))
+        if len(unit_spikes) >1:
+            all_ft_list.append(temporal_features(dp,unit_spikes, unit))
+        else:
+            all_ft_list.append([dp] + list(np.zeros(15)))
 
     return all_ft_list
 
@@ -1083,7 +1074,7 @@ def wvf_feat(dp, units):
     High level function for getting the wvf features from a single (integer) or
     set of units (list of units) from a dp dataset.
     """
-    if isinstance(units, int):
+    if isinstance(units, (int, np.int16, np.int32, np.int64)):
         units = [units]
 
     all_ft_list = []
@@ -1109,9 +1100,10 @@ def wvf_feat(dp, units):
             curr_feat.insert(0, dp)
             all_ft_list.append(curr_feat)
         else:
-            all_ft_list.append(0)
+            all_ft_list.append([dp]+[0]*17)
+            mean_pc = np.zeros(82)
 
-    return all_ft_list
+    return all_ft_list, mean_pc
 
 def temp_wvf_feat(dp, units):
     """
@@ -1129,35 +1121,41 @@ def temp_wvf_feat(dp, units):
             units = [int(units)]
         else:
             raise TypeError("Only ints, list of ints or ints disguised as floats allowed")
-#    breakpoint()
     all_feats = []
     for unit in units:
         t_feat = temp_feat(dp, unit)[0]
-        w_feat = wvf_feat(dp, unit)[0][2:]
-        all_feat = t_feat + w_feat
+        w_feat, mean_wvf = wvf_feat(dp, unit)
+        all_feat = t_feat + w_feat[0][2:]
         all_feats.append(all_feat)
-    all_feats = np.array((all_feats))
-    return all_feats
+#    all_feats = np.array((all_feats))
+    return all_feats, mean_wvf
 
 
-def get_pca_weights(all_acgs_matrix, n_components = 5, show = False):
+def get_pca_weights(all_acgs_matrix, n_components = 5, show = False, titl = 'WVF'):
     """
     Input: matrix with all the normalised acgs, size: n x m
     Return: matrix n x no_pca_feat
     """
 
-    X = StandardScaler().fit_transform(good_acgs)
+    X = StandardScaler().fit_transform(all_acgs_matrix)
     pca2 = PCA(n_components = n_components)
     projected = pca2.fit_transform(X)
     # show two plots
     #   - first one with the PCA features for the number of components
     #   - second is the variance explained per PC
     if show:
+
+        plt.figure()
+        plt.scatter(projected[:,0], projected[:,1] )
+        plt.title(f'Projection of first and second principal components for {titl}')
+        plt.xlabel('PC1')
+        plt.ylabel('PC2')
+
         pca_comp = pca2.components_
         plt.ion()
         plt.figure()
         line_objects = plt.plot(pca_comp.T )
-        plt.title(f'First {n_components} PC features, ACGs cut at {good_acgs.shape[1]}')
+        plt.title(f'First {n_components} PC features, {titl} vector len {all_acgs_matrix.shape[1]}')
         plt.legend(line_objects, tuple(np.arange(n_components)), title = 'PCA features')
 
         exp_var = np.round(np.sum(pca2.explained_variance_ratio_),3)
@@ -1166,7 +1164,7 @@ def get_pca_weights(all_acgs_matrix, n_components = 5, show = False):
         plt.plot(np.arange(n_components), pca2.explained_variance_ratio_)
         plt.xticks(np.arange(n_components),np.arange(n_components))
         plt.yticks(pca2.explained_variance_ratio_,np.round(pca2.explained_variance_ratio_, 2))
-        plt.title(f'Smoothed ACG, first {n_components} PCs and the variance explained by each,\n overall explain {exp_var} of variance, ACGs cut at {good_acgs.shape[1]}')
+        plt.title(f'{titl} first {n_components} PCs and the variance explained by each,\n overall explain {exp_var} of variance, {titl} vector len is {all_acgs_matrix.shape[1]}')
 
     return projected
 
